"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var couchbase_plugin_common_1 = require("./couchbase-plugin.common");
var types = require("tns-core-modules/utils/types");
var Couchbase = (function (_super) {
    __extends(Couchbase, _super);
    function Couchbase(databaseName) {
        var _this = _super.call(this, databaseName) || this;
        _this.ios = CBLDatabase.alloc().initWithNameError(databaseName);
        return _this;
    }
    Couchbase.prototype.createDocument = function (data, documentId) {
        var doc;
        if (documentId) {
            doc = CBLMutableDocument.alloc().initWithID(documentId);
        }
        else {
            doc = CBLMutableDocument.alloc().init();
        }
        var keys = Object.keys(data);
        for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
            var key = keys_1[_i];
            var item = data[key];
            doc.setValueForKey(item, key);
        }
        this.ios.saveDocumentError(doc);
        return doc.id;
    };
    Couchbase.prototype.getDocument = function (documentId) {
        var doc = this.ios.documentWithID(documentId);
        if (doc) {
            var obj = {};
            var keys = doc.keys;
            var size = keys.count;
            for (var i = 0; i < size; i++) {
                var key = keys.objectAtIndex(i);
                var value = doc.valueForKey(key);
                var newValue = {};
                newValue[key] = value;
                obj = Object.assign(obj, newValue);
            }
            return obj;
        }
        return null;
    };
    Couchbase.prototype.updateDocument = function (documentId, data) {
        var original = this.ios.documentWithID(documentId);
        var newDoc = original.toMutable();
        var keys = Object.keys(data);
        for (var _i = 0, keys_2 = keys; _i < keys_2.length; _i++) {
            var key = keys_2[_i];
            var item = data[key];
            newDoc.setValueForKey(item, key);
        }
        this.ios.saveDocumentError(newDoc);
    };
    Couchbase.prototype.deleteDocument = function (documentId) {
        var doc = this.ios.documentWithID(documentId);
        return this.ios.deleteDocumentError(doc);
    };
    Couchbase.prototype.destroyDatabase = function () {
        return this.ios.delete();
    };
    Couchbase.prototype.createPullReplication = function (remoteUrl, username, password) {
        var url = NSURL.alloc().initWithString(remoteUrl);
        var targetEndpoint = CBLURLEndpoint.alloc().initWithURL(url);
        var replConfig = CBLReplicatorConfiguration.alloc().initWithDatabaseTarget(this.ios, targetEndpoint);
        replConfig.replicatorType = CBLReplicatorType.kCBLReplicatorTypePull;
        if (username && password) {
            replConfig.authenticator = CBLBasicAuthenticator.alloc().initWithUsernamePassword(username, password);
        }
        var replicator = CBLReplicator.alloc().initWithConfig(replConfig);
        return new Replicator(replicator);
    };
    Couchbase.prototype.createPushReplication = function (remoteUrl, username, password) {
        var url = NSURL.alloc().initWithString(remoteUrl);
        var targetEndpoint = CBLURLEndpoint.alloc().initWithURL(url);
        var replConfig = CBLReplicatorConfiguration.alloc().initWithDatabaseTarget(this.ios, targetEndpoint);
        replConfig.replicatorType = CBLReplicatorType.kCBLReplicatorTypePush;
        if (username && password) {
            replConfig.authenticator = CBLBasicAuthenticator.alloc().initWithUsernamePassword(username, password);
        }
        var replicator = CBLReplicator.alloc().initWithConfig(replConfig);
        return new Replicator(replicator);
    };
    Couchbase.prototype.addDatabaseChangeListener = function (callback) {
        this.ios.addChangeListener(function (change) {
            if (callback && typeof callback === 'function') {
                var ids = [];
                var documentIds = change.documentIDs;
                var size = documentIds.count;
                for (var i = 0; i < size; i++) {
                    var item = documentIds[i];
                    ids.push(item);
                }
                callback(ids);
            }
        });
    };
    Couchbase.prototype.query = function (query) {
        if (query === void 0) { query = { select: [] }; }
        var items = [];
        var select = [];
        var from;
        var join = null;
        var where;
        var groupBy = null;
        var orderBy = null;
        var limit = null;
        var having = null;
        if (!query.select || query.select.length === 0) {
            select.push(CBLQuerySelectResult.all());
        }
        else {
            query.select.forEach(function (item) {
                if (item === couchbase_plugin_common_1.QueryMeta.ID) {
                    select.push(CBLQueryMeta.id);
                }
                else {
                    select.push(CBLQueryExpression.property(item));
                }
            });
        }
        if (query.from) {
            var db = new Couchbase(query.from);
            from = CBLQueryDataSource.database(db.ios);
        }
        else {
            from = CBLQueryDataSource.database(this.ios);
        }
        var nativeQuery = null;
        if (query.where) {
            for (var _i = 0, _a = query.where; _i < _a.length; _i++) {
                var item = _a[_i];
                if (item === couchbase_plugin_common_1.QueryLogicalOperator.AND) {
                    if (!nativeQuery)
                        break;
                }
                else if (item === couchbase_plugin_common_1.QueryLogicalOperator.OR) {
                    if (!nativeQuery)
                        break;
                }
                else {
                    if (item) {
                        switch (item.comparison) {
                            case 'equalTo':
                                nativeQuery = CBLQueryExpression.property(item.property).equalTo(CBLQueryExpression.value(item.value));
                                break;
                            case 'add':
                                break;
                            case 'between':
                                break;
                            case 'collate':
                                break;
                            case 'divide':
                                break;
                            case 'greaterThan':
                                nativeQuery = CBLQueryExpression.property(item.property).greaterThan(CBLQueryExpression.value(item.value));
                                break;
                            case 'greaterThanOrEqualTo':
                                nativeQuery = CBLQueryExpression.property(item.property).greaterThanOrEqualTo(CBLQueryExpression.value(item.value));
                                break;
                            case 'in':
                                var inArray = [];
                                if (Array.isArray(item.value)) {
                                    for (var _b = 0, _c = item.value; _b < _c.length; _b++) {
                                        var exp_1 = _c[_b];
                                        inArray.push(CBLQueryExpression.value(exp_1));
                                    }
                                }
                                else {
                                    inArray.push(CBLQueryExpression.value(item.value));
                                }
                                nativeQuery = CBLQueryExpression.property(item.property).in(inArray);
                                break;
                            case 'is':
                                nativeQuery = CBLQueryExpression.property(item.property).is(CBLQueryExpression.value(item.value));
                                break;
                            case 'isNot':
                                nativeQuery = CBLQueryExpression.property(item.property).isNot(CBLQueryExpression.value(item.value));
                                break;
                            case 'isNullOrMissing':
                                nativeQuery = CBLQueryExpression.property(item.property).isNullOrMissing();
                                break;
                            case 'lessThan':
                                nativeQuery = CBLQueryExpression.property(item.property).lessThan(CBLQueryExpression.value(item.value));
                                break;
                            case 'lessThanOrEqualTo':
                                nativeQuery = CBLQueryExpression.property(item.property).lessThanOrEqualTo(CBLQueryExpression.value(item.value));
                                break;
                            case 'like':
                                nativeQuery = CBLQueryExpression.property(item.property).like(CBLQueryExpression.value(item.value));
                                break;
                            case 'modulo':
                                nativeQuery = CBLQueryExpression.property(item.property).modulo(CBLQueryExpression.value(item.value));
                                break;
                            case 'multiply':
                                nativeQuery = CBLQueryExpression.property(item.property).multiply(CBLQueryExpression.value(item.value));
                                break;
                            case 'notEqualTo':
                                nativeQuery = CBLQueryExpression.property(item.property).notEqualTo(CBLQueryExpression.value(item.value));
                                break;
                            case 'notNullOrMissing':
                                nativeQuery = CBLQueryExpression.property(item.property).notNullOrMissing();
                                break;
                            case 'regex':
                                nativeQuery = CBLQueryExpression.property(item.property).regex(CBLQueryExpression.value(item.value));
                                break;
                        }
                    }
                }
            }
        }
        if (query.groupBy) {
            if (query.groupBy.length > 0) {
                groupBy = [];
            }
            for (var _d = 0, _e = query.groupBy; _d < _e.length; _d++) {
                var prop = _e[_d];
                groupBy.push(CBLQueryExpression.property(prop));
            }
        }
        if (query.order) {
            if (query.order.length > 0) {
                orderBy = [];
            }
            for (var _f = 0, _g = query.order; _f < _g.length; _f++) {
                var item = _g[_f];
                switch (item.direction) {
                    case 'desc':
                        orderBy.push(CBLQueryOrdering.property(item.property).descending());
                        break;
                    default:
                        orderBy.push(CBLQueryOrdering.property(item.property).ascending());
                        break;
                }
            }
        }
        if (query.limit && typeof query.limit === 'number') {
            limit = CBLQueryLimit.limit(CBLQueryExpression.value(query.limit));
        }
        var queryBuilder = CBLQueryBuilder.selectFromJoinWhereGroupByHavingOrderByLimit(select, from, join, nativeQuery, groupBy, having, orderBy, limit);
        var result = queryBuilder.execute().allResults();
        var size = result.count;
        for (var i = 0; i < size; i++) {
            var item = result.objectAtIndex(i);
            var keys = item.keys;
            var keysSize = keys.count;
            var obj = {};
            for (var keyId = 0; keyId < keysSize; keyId++) {
                var key = keys.objectAtIndex(keyId);
                var nativeItem = item.valueForKey(key);
                if (types.getClass(nativeItem) === 'CBLDictionary') {
                    var cblKeys = nativeItem.keys;
                    var cblKeysSize = cblKeys.count;
                    for (var cblKeysId = 0; cblKeysId < cblKeysSize; cblKeysId++) {
                        var cblKey = cblKeys.objectAtIndex(cblKeysId);
                        obj[cblKey] = nativeItem.valueForKey(cblKey);
                    }
                }
            }
            items.push(obj);
        }
        return items;
    };
    return Couchbase;
}(couchbase_plugin_common_1.Common));
exports.Couchbase = Couchbase;
var Replicator = (function (_super) {
    __extends(Replicator, _super);
    function Replicator(replicator) {
        return _super.call(this, replicator) || this;
    }
    Replicator.prototype.start = function () {
        this.replicator.start();
    };
    Replicator.prototype.stop = function () {
        this.replicator.stop();
    };
    Replicator.prototype.isRunning = function () {
        return (this.replicator.status.activity ===
            CBLReplicatorActivityLevel.kCBLReplicatorBusy);
    };
    Replicator.prototype.setContinuous = function (isContinuous) {
        this.replicator.config.continuous = isContinuous;
    };
    return Replicator;
}(couchbase_plugin_common_1.ReplicatorBase));
exports.Replicator = Replicator;
//# sourceMappingURL=couchbase-plugin.ios.js.map